/*
 * file ListClassesVisitor.java
 *
 * IBM Confidential
 * OCO Source Materials
 * Rational ClearCase
 * (C) Copyright IBM Corp. 2010.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.rational.wvcm.stpex.annotation.processing;

import static com.sun.mirror.util.DeclarationVisitors.NO_OP;
import static com.sun.mirror.util.DeclarationVisitors.getSourceOrderDeclarationScanner;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;


import com.ibm.rational.wvcm.stpex.annotation.RpcEnum;
import com.ibm.rational.wvcm.stpex.annotation.processing.StpVisitor.TagInfo;
import com.sun.mirror.apt.AnnotationProcessorEnvironment;
import com.sun.mirror.apt.Filer;
import com.sun.mirror.declaration.AnnotationMirror;
import com.sun.mirror.declaration.AnnotationTypeElementDeclaration;
import com.sun.mirror.declaration.AnnotationValue;
import com.sun.mirror.declaration.ClassDeclaration;
import com.sun.mirror.type.AnnotationType;
import com.sun.mirror.util.SimpleDeclarationVisitor;

public class ListClassesVisitor extends SimpleDeclarationVisitor 
{
    static final String INDENT = "    ";
    static final String ID_NAME = "id";
    static final String NS_NAME = "ns";
    static final String SIMPLE_NAME = "simpleName";
    
    private final AnnotationProcessorEnvironment m_env;
    
    public ListClassesVisitor(AnnotationProcessorEnvironment env)
    {
        //System.out.println("Creating ListClassesVisitor");
        m_env = env;
    }

    @Override
    public void visitClassDeclaration(ClassDeclaration c) 
    {
        //System.out.println("Visiting Class: " + c.getQualifiedName());
        
        StpVisitor visitor = null;
        
        if (c.getQualifiedName().equals("com.ibm.rational.stp.cs.internal.util.XmlTag")) {
        
            visitor =  new XmlTagVisitor(m_env);
        }
        else if (c.getQualifiedName().equals("com.ibm.rational.stp.cs.internal.util.XmlNs")) {
            
            visitor =  new XmlNsVisitor(m_env);
        }
        else if (c.getQualifiedName().equals("com.ibm.rational.wvcm.stp.StpProvider.Domain")) {
            
            visitor = new ProviderDomainVisitor(m_env);
        }
        else if (c.getQualifiedName().equals("com.ibm.rational.wvcm.stp.StpLocation.Namespace")) {
            
            visitor = new StpLocationNamespaceVisitor(m_env);
        }
        else if (c.getQualifiedName().equals("com.ibm.rational.wvcm.stp.StpException.StpReasonCode")) {
            
            visitor = new StpExceptionStpReasonCodeVisitor(m_env);
        }
        
        if (visitor != null) {
            visitor.begin();
            c.accept(getSourceOrderDeclarationScanner(NO_OP, (SimpleDeclarationVisitor)visitor));
            visitor.end();
        }
    }
    
    static TagInfo getRpcEnumValues(Collection<AnnotationMirror> amirrors)
    {
        for (AnnotationMirror am : amirrors) {

            try {
                String name = am.getAnnotationType().getDeclaration().getQualifiedName();
                
                if (name.equals(RpcEnum.class.getName())) {
                    return getRpcValues(am);
                }
            }
            catch (ClassCastException e) {

                System.err.println("Error: encoutnered unknown annotation" + am.getPosition().toString());
            }
        }
        
        return null;
    }
    
    static TagInfo getRpcValues(AnnotationMirror am) 
    {
        TagInfo info = new TagInfo();
        
        Map<AnnotationTypeElementDeclaration, AnnotationValue> values = am.getElementValues();

        info.position = am.getPosition().toString();
        
        for (Entry<AnnotationTypeElementDeclaration, AnnotationValue> entry : values.entrySet()) {
            
            if (entry.getKey().getSimpleName().equals(ID_NAME)) {
                info.id = entry.getValue().toString();
            }
            else if (entry.getKey().getSimpleName().equals(NS_NAME)) {
                info.ns = (String)entry.getValue().getValue();
            }
            else if (entry.getKey().getSimpleName().equals(SIMPLE_NAME)) {
                info.simple = (String)entry.getValue().getValue();
            }
        }
        
        return info;
    }
    
    static void ensureUniqueAndAddIfSo(Map<Integer, TagInfo> map, TagInfo info) 
    {
        int intValue = Integer.parseInt(info.id);
        
        if (map.containsKey(intValue)) {
            
            System.err.println("Error: For enum '" + info.fieldName + 
                               "' RpcEnumId value '" + intValue + 
                               "' is already defined for enum '" + 
                               map.get(intValue).fieldName); 
            System.err.println("Existing pos: " + info.position);
            System.err.println("Current pos: " + info.position);
            System.err.println("Error: exiting unexpectedly");
            System.exit(1);
        }
        else {
            map.put(intValue, info);
        }
    }
    
    static PrintWriter createFile(Filer filer, String fileName)
    {
        System.out.println("Creating file: " + fileName);
        try {
            return filer.createTextFile(Filer.Location.SOURCE_TREE, 
                                        /*package*/"", 
                                        new File(fileName), 
                                        /*charset*/null);
        }
        catch (IOException e) {
            e.printStackTrace();
            System.err.println("Error: exiting unexpectedly");
            System.exit(1);
        }
        
        return null;
    }
    
    static void writeCommonHeader(PrintWriter pw, Object o)
    {
        pw.println("/**");
        pw.println(" * Do not modify this file. This is a generated file");
        pw.println(" * Generated By: " + o.getClass().getName());
        pw.println(" */");
    }
}
